"""
Enterprise data api application. This Django app exposes API endpoints used by enterprises.
"""

__version__ = "4.6.10"


default_app_config = "enterprise_data.apps.EnterpriseDataAppConfig"  # pylint: disable=invalid-name
