# Generated by Django 2.2.17 on 2021-09-16 04:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0025_auto_20210703_1854'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriselearnerenrollment',
            name='primary_program_type',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='contract_id',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='coupon_code',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='coupon_name',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='course_primary_program',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='course_primary_subject',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='letter_grade',
            field=models.CharField(max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='offer_type',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='paid_by',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='progress_status',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='seat_delivery_method',
            field=models.CharField(max_length=255, null=True),
        ),
    ]
